#pragma once

#define BRACKETS_THAT(that) (that)

#ifndef MAX
#define MAX(a, b) ((a) < (b) ? (b) : (a))
#endif /* MAX */

#ifndef MIN
#define MIN(a, b) ((a) > (b) ? (b) : (a))
#endif /* MIN */

#define RETURN_IF(cond, msg, ret)                                       \
    if (cond) {                                                         \
        LOG_ERROR("%s", msg);                                           \
        return ret;                                                     \
    }
#define GOTO_IF(cond, msg, label)                                       \
    if (cond) {                                                         \
        LOG_ERROR("%s", msg);                                           \
        goto label;                                                     \
    }

#define GOTO_UNLESS(cond, msg, label)   GOTO_IF(!(cond), msg, label)
#define RETURN_UNLESS(cond, msg, ret)   RETURN_IF(!(cond), msg, ret)
#define NOTHING                         /* Usefull to return nothing. */

#define STRTOL(ret, str, endptr, err_flag)                              \
{                                                                       \
    err_flag = 0;                                                       \
    errno    = 0;                                                       \
    ret      = str == NULL ? 0 : strtol(str, &(endptr), 0);             \
    err_flag = errno != 0 || endptr == str;                             \
}

#define ERROR   1
#define WARN    2
#define INFO    3
#define DEBUG   4
#define LOG(format, level, section, ...)                                            \
    fprintf(stderr, " %c %s%s: " format "\n",                                       \
            level == ERROR ? '!' : level == WARN ? '*' : level == INFO ? '.' : ' ', \
            sizeof(section) > sizeof("") ? ("[" section "] ") : "",                 \
            __func__,                                                               \
            __VA_ARGS__)
#define LOG_INFO_SCT(section, format, ...)      LOG(format, INFO, section, __VA_ARGS__)
#define LOG_WARN_SCT(section, format, ...)      LOG(format, WARN, section, __VA_ARGS__)
#define LOG_ERROR_SCT(section, format, ...)     LOG(format, ERROR, section, __VA_ARGS__)
#define LOG_DEBUG_SCT(section, format, ...)     LOG(format, DEBUG, section, __VA_ARGS__)
#define LOG_INFO(format, ...)                   LOG_INFO_SCT("", format, __VA_ARGS__)
#define LOG_WARN(format, ...)                   LOG_WARN_SCT("", format, __VA_ARGS__)
#define LOG_ERROR(format, ...)                  LOG_DEBUG_SCT("", format, __VA_ARGS__)
#define LOG_DEBUG(format, ...)                  LOG_DEBUG_SCT("", format, __VA_ARGS__)
