#pragma once

#include <common/macro.h>
#include <common/define.h>
#include <stdint.h>
#include <stdlib.h>

/* Defines */

#define URL_MAX_LEN     1024
#define DEFAULT_URL     "https://kurisu.iiens.net"
#define GET_ID_JSON     DEFAULT_URL "/api_karas.php?id=%ld"
#define GET_ID_FILE     DEFAULT_URL "/download.php?id=%ld"

#define SELF_EXECUTABLE_LINUX           "/proc/self/exe"
#define SELF_EXECUTABLE_FREEBSD         "/proc/curproc/file"
#define SELF_EXECUTABLE_SOLARIS         "/proc/self/path/a.out"


#define LKT_MAX_SQLITE_STATEMENT        1024
#define PROTECTED_DATABASE              "disk"

#define LKT_DATABASE_NAME_KID           "id"
#define LKT_DATABASE_NAME_KNAME         "source_name"
#define LKT_DATABASE_NAME_KTITLE        "song_title"
#define LKT_DATABASE_NAME_KCAT          "category"
#define LKT_DATABASE_NAME_KTYPE         "song_type"
#define LKT_DATABASE_NAME_KAUTHOR       "author_name"
#define LKT_DATABASE_NAME_KAUTHOR_YEAR  "author_year"
#define LKT_DATABASE_NAME_KLANG         "language"
#define LKT_DATABASE_KARA_COLUMNT_ANY   "any_col"
#define LKT_DATABASE_KARA_ALL           "string"

#define LEKTOR_TAG_MAX                  256
#define LKT_MESSAGE_ARGS_MAX            32
#define LKT_MESSAGE_MAX                 2048
#define LKT_DEFAULT_LIST_SIZE           10

typedef volatile enum {
    MPD_IDLE_NONE               = 0,

    MPD_IDLE_DATABASE           = ( 1 << 1  ),
    MPD_IDLE_UPDATE             = ( 1 << 2  ),
    MPD_IDLE_STORED_PLAYLIST    = ( 1 << 3  ),
    MPD_IDLE_PLAYLIST           = ( 1 << 4  ),
    MPD_IDLE_PLAYER             = ( 1 << 5  ),
    MPD_IDLE_MIXER              = ( 1 << 6  ),
    MPD_IDLE_OUTPUT             = ( 1 << 7  ),
    MPD_IDLE_OPTIONS            = ( 1 << 8  ),
    MPD_IDLE_PARTITION          = ( 1 << 9  ),
    MPD_IDLE_STICKER            = ( 1 << 10 ),
    MPD_IDLE_SUBSCRIPTION       = ( 1 << 11 ),
    MPD_IDLE_MESSAGE            = ( 1 << 12 ),

    MPD_IDLE_ALL =
        MPD_IDLE_DATABASE | MPD_IDLE_UPDATE | MPD_IDLE_STORED_PLAYLIST  |
        MPD_IDLE_PLAYLIST | MPD_IDLE_PLAYER | MPD_IDLE_MIXER            |
        MPD_IDLE_OUTPUT | MPD_IDLE_OPTIONS | MPD_IDLE_PARTITION         |
        MPD_IDLE_STICKER | MPD_IDLE_SUBSCRIPTION | MPD_IDLE_MESSAGE,
} mpd_idle_flag;

#define LKT_BACKLOG         32
#define COMMAND_LIST_MAX    64
#define BUFFER_OUT_MAX      16
#define MPD_VERSION         "0.21.16"

/* Macros */

#define SQLITE_PREPARE(db, stmt, SQL, goto_label)                               \
    if (sqlite3_prepare_v2((sqlite3 *) db, SQL, -1, &(stmt), 0) != SQLITE_OK) { \
        LOG_ERROR_SCT("DB", "Failed to prepare statement: %s",                  \
                      sqlite3_errmsg((sqlite3 *) db));                          \
        goto goto_label;                                                        \
    }

#define SQLITE_EXEC(db, SQL, goto_label)                                        \
    if (sqlite3_exec((sqlite3 *) db, SQL, NULL, NULL, NULL) != SQLITE_OK) {     \
        LOG_ERROR_SCT("DB", "Failed to exec statement: %s",                     \
                      sqlite3_errmsg((sqlite3 *) db));                          \
        goto goto_label;                                                        \
    }

#define SQLITE_BIND_TEXT(db, stmt, pos, text, error)                    \
    if (sqlite3_bind_text(stmt, pos, text, -1, 0) != SQLITE_OK) {       \
        LOG_ERROR_SCT("DB", "Failed to bind text %s at pos %d: %s",     \
                      text, pos, sqlite3_errmsg((sqlite3 *) db));       \
        goto error;                                                     \
    }

#define SQLITE_BIND_INT(db, stmt, pos, integer, error)                  \
    if (sqlite3_bind_int(stmt, pos, integer) != SQLITE_OK) {            \
        LOG_ERROR_SCT("DB", "Failed to bind int %d at pos %d: %s",      \
                      integer, pos, sqlite3_errmsg((sqlite3 *) db));    \
        goto error;                                                     \
    }

#define SQLITE_STEP(db, stmt, code, error)                              \
    if (sqlite3_step(stmt) != code) {                                   \
        LOG_ERROR_SCT("DB", "Failed to step: %s",                       \
                      sqlite3_errmsg((sqlite3 *) db));                  \
        goto error;                                                     \
    }

#define SQLITE_STEP_ROW(db, stmt, error)    SQLITE_STEP(db, stmt, SQLITE_ROW, error)
#define SQLITE_STEP_DONE(db, stmt, error)   SQLITE_STEP(db, stmt, SQLITE_DONE, error)
#define SQLITE_STEP_OK(db, stmt, error)     SQLITE_STEP(db, stmt, SQLITE_OK, error)

#define SQLITE_DO_ROLLBACK(db)                                          \
    sqlite3_exec((sqlite3 *) db, "ROLLBACK TRANSACTION;\n", NULL, NULL, NULL);

/* Random things */

/* Read `bytes` as the big endian representation of a 32-bit unsigned integer.
   `n` is the number of bytes in `bytes`
   Returns 0 if n is 0.
   Restriction: n <= 4 */
uint32_t be_uint32_t(const uint8_t bytes[], size_t n);

/* Same as `be_uint32_t` but for 64-bit unsigned integers.
   Restriction: n <= 8 */
uint64_t be_uint64_t(const uint8_t bytes[], size_t n);

/* Trim the string `str` of the char `c` from the right and the left.
   The string may not be null terminated.
   Returns a pointer to the the trimmed string
   Restrictions: the len of the string must be `len`. */
char *trim(char *str, size_t len, char c);

/* Get a line from stdin safely. */
int get_stdin_line(const char *prompt, char *buf, size_t len);
