#pragma once

#include <lektor/common.h>
#include <stddef.h>
#include <sqlite3.h>

static const char *const lkt_default_config_file =
    "; Lektor's configuration file\n"
    "; This file was generated, you may change it to your needs\n"
    "\n"
    "; Externals executables that may be called by lektord or any\n"
    "; generated binary.\n"
    "[externals]\n"
    "mkvpropedit = /usr/bin/mkvpropedit\n"
    "sqlite3 = /usr/bin/sqlite3\n"
    "\n"
    "; Server configuration.\n"
    "[server]\n"
    "host = 0.0.0.0      ; Listen any incomming connexions\n"
    "port = 6600         ; Default MPD port\n"
    "max_clients = 16    ; Maximun handled clients\n"
    "\n"
    "; Database configuration.\n"
    "[database]\n"
    "kara_dir = /home/kara                          ; Root for the mkv files\n"
    "db_path = /home/kara/kara.db                   ; Sqlite3 database, must be rw\n"
    "init_script = LKT_PREFIX/share/lektor/init.sql ; Init script for the database\n"
    "\n"
    "; Player configuration, the window where the karas\n"
    "; can be seen.\n"
    "[player]\n"
    "autoclear = true            ; Clear the queue when starting\n"
    "module = module_winx11      ; Default module for the window\n"
    "def_random = false          ; Play at random per default\n"
    "def_consume = false         ; Consume per default\n"
    "def_single = false          ; Single per default\n"
    "def_repeat = false          ; Repeat per default\n"
    "\n"
    "; The WinX11 module configuration. Modules are .so files\n"
    "; loaded by lektor.\n"
    "[module_x11]\n"
    "path = LKT_PREFIX/lib/lektor/lib_module_x11.so\n"
    "load_function = module_set_function\n"
    "xwallpaper = false\n"
    "\n"
    "[module_sdl2]\n"
    "path = LKT_PREFIX/lib/lektor/lib_module_sdl2.so\n"
    "load_function = module_set_function\n";

/* It is just an alias to the consecutive use of config_detect_file and
   config_new function, with the return an '&&' between the two returns of the
   two other functions. */
int config_open(volatile sqlite3 *db);

/* Get the path to the config file that may be red, taking into account the
   priority between existing files. The returned path is a path to an existing
   file. If no file is found, returns a non zero value. Returns 1 otherwise. */
int config_detect_file(char *conf, size_t conf_len);

/* Create and read the configuration in the conf file and write it into
   lkt_conf. The type is opaque because it's a C++ class.
   Return 0 if the read operation was a success, a non zero value otherwise. */
int config_new(volatile sqlite3 *db, const char *conf);

/* Load a module by its name. Also initialize the mod pointer. No check on data
   type is done, it is up to the user to check if the structure passed as a
   `void*` is the right structure. */
int load_module_by_name(volatile sqlite3 *db, const char *name, void *mod);
