#define _POSIX_C_SOURCE 200809L

#include <mthread/mthread_internal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Functions for handling conditional variables.  */

/* Initialize condition variable COND using attributes ATTR, or use
   the default values if later is NULL.  */
int
mthread_cond_init(mthread_cond_t *__cond, const mthread_condattr_t *__cond_attr)
{
    (void)__cond_attr;
    __cond->lock = 0;
    __cond->list = safe_malloc(sizeof(struct mthread_list_s));
    memset(__cond->list, 0, sizeof(struct mthread_list_s));
    LOG_INFO("%s", "Successfully created a condition");
    return 0;
}

/* Destroy condition variable COND.  */
int
mthread_cond_destroy(mthread_cond_t *__cond)
{
    if (__cond->list)
        free((void *) __cond->list);
    return 0;
}

/* Wake up one thread waiting for condition variable COND.  */
int
mthread_cond_signal(mthread_cond_t *__cond)
{
    mthread_t first, self = mthread_self();
    mthread_virtual_processor_t *vp;
    int ret = 1;
    mthread_spinlock_lock(&__cond->lock);

    if (__cond->list->first != NULL) {
        first = mthread_remove_first(__cond->list);
        vp = mthread_get_vp();
        first->status = RUNNING;
        mthread_insert_last(first, &(vp->ready_list));
        LOG_INFO("Wake up the thread %p from the thread %p", (void *) first, (void *) self);
        ret = 0;
    } else
        LOG_INFO("No thread to wake up found from the thread %p", (void *) self);

    mthread_spinlock_unlock(&__cond->lock);
    return ret;
}

/* Wake up all threads waiting for condition variables COND.  */
int
mthread_cond_broadcast(mthread_cond_t *__cond)
{
    mthread_t first, self = mthread_self();
    mthread_virtual_processor_t *vp;
    mthread_spinlock_lock(&__cond->lock);

    while (__cond->list->first != NULL) {
        first = mthread_remove_first(__cond->list);
        vp = mthread_get_vp();
        first->status = RUNNING;
        LOG_INFO("Wake up the thread %p from the thread %p", (void *) first, (void *) self);
        mthread_insert_last(first, &(vp->ready_list));
    }

    mthread_spinlock_unlock(&__cond->lock);
    return 0;
}

/* Wait for condition variable COND to be signaled or broadcast.
   MUTEX is assumed to be locked before.  */
int
mthread_cond_wait(mthread_cond_t *__cond, mthread_mutex_t *__mutex)
{
    mthread_spinlock_lock(&__cond->lock);
    mthread_t self = mthread_self();
    mthread_virtual_processor_t *vp = mthread_get_vp();
    mthread_insert_last(self, __cond->list);
    self->status = BLOCKED;
    mthread_mutex_unlock(__mutex);

    LOG_ERROR("Waiting in condition for thread %p", (void *) self);

    vp->p = &__cond->lock;
    mthread_yield();

    mthread_mutex_lock(__mutex);
    LOG_ERROR("Waike up in condition for thread %p", (void *) self);

    return 0;
}
